/*
								+---------------------------------+
								|                                 |
								|   ***   Advanced effect   ***   |
								|                                 |
								|  Copyright   -tHE SWINe- 2006  |
								|                                 |
								|            Effect.h             |
								|                                 |
								+---------------------------------+
*/

#ifndef __EFFECT2_INCLUDED
#define __EFFECT2_INCLUDED

/*
 *	class CEffect2
 *		- class for advanced demo effect
 */
class CEffect2 {
public:
	/*
	 *	virtual ~CEffect2()
	 *		- default destructor (supposed to call Shutdown())
	 */
	virtual ~CEffect2() {}

	/*
	 *	virtual const char *p_s_Name()
	 *		- returns short effect name (c-like identifier, name of effect class for example)
	 */
	virtual const char *p_s_Name() = 0;

	/*
	 *	virtual const char *p_s_LongName()
	 *		- returns full effect name
	 */
	virtual const char *p_s_LongName() = 0;

	/*
	 *	virtual bool b_Supported()
	 *		- returns true in case effect is supported by OpenGL, otherwise false
	 */
	virtual bool b_Supported() = 0;

	/*
	 *	virtual bool Init(CGLState *p_state, int n_width, int n_height)
	 *		- loads all effect resources, creates OpenGL buffers, compiles shaders, etc
	 *		- returns true on success, false on failure
	 */
	virtual bool Init(CGLState *p_state, /*CLoggerDialog &r_logwnd,*/ int n_width, int n_height) = 0;

	/*
	 *	virtual bool b_Status()
	 *		- returns true in case effect is loaded and ready for rendering, otherwise false
	 */
	virtual bool b_Status() = 0;

	/*
	 *	virtual bool Shutdown(CGLState *p_state)
	 *		- shutdown effect
	 *		- returns true in case no errors occured (even in case effect was not initialized)
	 *		  otherwise returns false
	 */
	virtual bool Shutdown(CGLState *p_state) = 0;

	/*
	 *	virtual float f_Length()
	 *		- returns effect length in seconds
	 */
	virtual float f_Length() = 0;

	/*
	 *	virtual bool Render(CGLState *p_state, float f_time)
	 *		- renders a single frame into current framebuffer; f_time is frame time,
	 *		  it can be out of <0, f_Length()> range, it's up to effect on how to handle that
	 *		  (repeat itself, fade out, render nothing ...)
	 */
	virtual bool Render(CGLState *p_state, float f_time) = 0;
};

/*
 *	class CSmokeBackground
 *		- simple background effect with smoke particles
 */
class CSmokeBackground : public CEffect2 {
protected:
	CGLTexture_2D *m_p_smoke_texture;

public:
	/*
	 *	CSmokeBackground()
	 *		- default constructor
	 */
	CSmokeBackground();

	/*
	 *	virtual ~CSmokeBackground()
	 *		- default destructor (supposed to call Shutdown())
	 */
	virtual ~CSmokeBackground();

	/*
	 *	virtual const char *p_s_Name()
	 *		- returns short effect name (c-like identifier, name of effect class for example)
	 */
	virtual const char *p_s_Name();

	/*
	 *	virtual const char *p_s_LongName()
	 *		- returns full effect name
	 */
	virtual const char *p_s_LongName();

	/*
	 *	virtual bool b_Supported()
	 *		- returns true in case effect is supported by OpenGL, otherwise false
	 */
	virtual bool b_Supported();

	/*
	 *	virtual bool Init(CGLState *p_state, int n_width, int n_height)
	 *		- loads all effect resources, creates OpenGL buffers, compiles shaders, etc
	 *		- returns true on success, false on failure
	 */
	virtual bool Init(CGLState *p_state, /*CLoggerDialog &r_logwnd,*/ int n_width, int n_height);

	/*
	 *	virtual bool b_Status()
	 *		- returns true in case effect is loaded and ready for rendering, otherwise false
	 */
	virtual bool b_Status();

	/*
	 *	virtual bool Shutdown(CGLState *p_state)
	 *		- shutdown effect
	 *		- returns true in case no errors occured (even in case effect was not initialized)
	 *		  otherwise returns false
	 */
	virtual bool Shutdown(CGLState *p_state);

	/*
	 *	virtual float f_Length()
	 *		- returns effect length in seconds
	 */
	virtual float f_Length();

	/*
	 *	virtual bool Render(CGLState *p_state, float f_time)
	 *		- renders a single frame into current framebuffer; f_time is frame time,
	 *		  it can be out of <0, f_Length()> range, it's up to effect on how to handle that
	 *		  (repeat itself, fade out, render nothing ...)
	 */
	virtual bool Render(CGLState *p_state, float f_time);
};

/*
 *	class CHexLattice
 *		- hexagonal rods, sliding up and down
 */
class CHexLattice : public CEffect2 {
protected:
	CGLTexture_2D *m_p_rod_top_diffuse;
	CGLTexture_2D *m_p_rod_top_specular;
	CGLTexture_2D *m_p_rod_top_local;
	CGLTexture_2D *m_p_rod_side_diffuse;
	CGLTexture_2D *m_p_rod_side_specular;
	CGLTexture_2D *m_p_rod_side_local;

	CGLFrameBuffer_FBO *m_p_render_buffer;
	CGLTexture_2D *m_p_render_texture[2]; // image is first rendered into those, it's post-processed then

	struct TVertex {
		Vector3f v_position;
		Vector3f v_normal;
		Vector3f v_tangent, v_bitangent;
		Vector2f v_texcoord;
	};

	// GL_T2F_C4F_N3F_V3F (size 12)

	float m_p_rod_vertex_array[12 * 30];
	static const int m_p_rod_index_array_top[]; // GL_TRIANGLE_FAN
	static const int m_p_rod_index_array_side[]; // GL_QUADS

	static const float f_rod_radius;
	static const float f_rod_height;

	static const bool m_b_enable_mipmaps;

	CGLShaderObject *m_p_local_bump_shader, *m_p_postprocessing_shader;
	/*static TShaderInfo::TShader m_t_vs_high, m_t_fs_high, m_t_fs_high2;
	static TShaderInfo::TShader::TSamplerInfo m_p_fs_samplers[3], m_p_fs_samplers2[1];*/
	static TShaderInfo m_t_vertex_shader;
	static TShaderInfo m_t_fragment_shader, m_t_fragment_shader2;

	CShaderVectorParameterRef<float> m_t_direction_param; // postprocessing shader parameter

	bool m_b_dl_ready;
	float m_f_aspect;

	int m_n_width, m_n_height;

public:
	/*
	 *	CHexLattice()
	 *		- default constructor
	 */
	CHexLattice();

	/*
	 *	virtual ~CHexLattice()
	 *		- default destructor (supposed to call Shutdown())
	 */
	virtual ~CHexLattice();

	/*
	 *	virtual const char *p_s_Name()
	 *		- returns short effect name (c-like identifier, name of effect class for example)
	 */
	virtual const char *p_s_Name();

	/*
	 *	virtual const char *p_s_LongName()
	 *		- returns full effect name
	 */
	virtual const char *p_s_LongName();

	/*
	 *	virtual bool b_Supported()
	 *		- returns true in case effect is supported by OpenGL, otherwise false
	 */
	virtual bool b_Supported();

	/*
	 *	virtual bool Init(CGLState *p_state, int n_width, int n_height)
	 *		- loads all effect resources, creates OpenGL buffers, compiles shaders, etc
	 *		- returns true on success, false on failure
	 */
	virtual bool Init(CGLState *p_state, /*CLoggerDialog &r_logwnd,*/ int n_width, int n_height);

	/*
	 *	virtual bool b_Status()
	 *		- returns true in case effect is loaded and ready for rendering, otherwise false
	 */
	virtual bool b_Status();

	/*
	 *	virtual bool Shutdown(CGLState *p_state)
	 *		- shutdown effect
	 *		- returns true in case no errors occured (even in case effect was not initialized)
	 *		  otherwise returns false
	 */
	virtual bool Shutdown(CGLState *p_state);

	/*
	 *	virtual float f_Length()
	 *		- returns effect length in seconds
	 */
	virtual float f_Length();

	/*
	 *	virtual bool Render(CGLState *p_state, float f_time)
	 *		- renders a single frame into current framebuffer; f_time is frame time,
	 *		  it can be out of <0, f_Length()> range, it's up to effect on how to handle that
	 *		  (repeat itself, fade out, render nothing ...)
	 */
	virtual bool Render(CGLState *p_state, float f_time);

protected:
	void Generate_RodVertices();
	void Draw_Rod(CGLState *p_state, int n_x, int n_z, float f_y);
	bool Render_Texture(CGLState *p_state, float f_time);
};

#endif //__EFFECT2_INCLUDED
