/*
								+---------------------------------+
								|                                 |
								| ***  Multi-mon win support  *** |
								|                                 |
								|  Copyright   -tHE SWINe- 2006  |
								|                                 |
								|          Monitors.cpp           |
								|                                 |
								+---------------------------------+
*/

#include "../UberLame_src/NewFix.h"
#include "../UberLame_src/CallStack.h"
#include <windows.h>
#define COMPILE_MULTIMON_STUBS
#include <multimon.h>
#include <string.h>
#include <stdio.h>
#include "Monitors.h"

#if defined(_MSC_VER) && !defined(__MWERKS__) && !defined(for)
#define for if(0) {} else for
#endif

TMultimonInfo::TMultimonInfo()
	:m_n_monitor_num(0), m_p_monitor(0)
{
}

TMultimonInfo::~TMultimonInfo()
{
	if(m_p_monitor)
		delete[] m_p_monitor;
}

bool TMultimonInfo::Update_Monitors()
{
	if(m_p_monitor) {
		delete[] m_p_monitor;
		m_p_monitor = 0;
	}
	m_n_monitor_num = 0;

	if((m_n_monitor_num = GetSystemMetrics(SM_CMONITORS)) <= 1) {
		m_n_monitor_num = 1;
		if(!(m_p_monitor = new(std::nothrow) TMonitorInfo)) {
			m_n_monitor_num = 0;
			return false;
		}
		strcpy(m_p_monitor->p_s_name, "primary monitor");
		m_p_monitor->b_valid = true;
		m_p_monitor->b_on = true;
		m_p_monitor->n_x = 0;
		m_p_monitor->n_y = 0;
		m_p_monitor->n_width = GetSystemMetrics(SM_CXSCREEN);
		m_p_monitor->n_height = GetSystemMetrics(SM_CYSCREEN);
		m_t_full_window = *m_p_monitor;
		m_t_full_on_window = *m_p_monitor;
		return true;
	}
	// single monitor

	strcpy(m_t_full_window.p_s_name, "virtual screen");
	m_t_full_window.b_valid = true;
	m_t_full_window.b_on = true;
	m_t_full_window.n_x = GetSystemMetrics(SM_XVIRTUALSCREEN);
	m_t_full_window.n_y = GetSystemMetrics(SM_YVIRTUALSCREEN);
	m_t_full_window.n_width = GetSystemMetrics(SM_CXVIRTUALSCREEN);
	m_t_full_window.n_height = GetSystemMetrics(SM_CYVIRTUALSCREEN);
	// right?

	if(!(m_p_monitor = new(std::nothrow) TMonitorInfo[m_n_monitor_num])) {
		m_n_monitor_num = 0;
		return false;
	}

	m_t_full_on_window.b_valid = true;
	m_t_full_on_window.b_on = false;
	m_t_full_on_window.n_x = 0;
	m_t_full_on_window.n_y = 0;
	m_t_full_on_window.n_width = 0;
	m_t_full_on_window.n_height = 0;
	// switch off

	for(int i = 0; i < m_n_monitor_num; i ++)
		m_p_monitor[i].b_valid = false;

	EnumDisplayMonitors(0, 0, MonCallback, (int)this);
	// enum monitors

	for(int i = 0; i < m_n_monitor_num; i ++) {
		if(m_p_monitor[i].b_valid) {
			if(!m_t_full_on_window.b_on) {
				m_t_full_on_window = m_p_monitor[i];
				m_t_full_on_window.n_width += m_t_full_on_window.n_x;
				m_t_full_on_window.n_height += m_t_full_on_window.n_y;
			} else {
				if(m_t_full_on_window.n_x > m_p_monitor[i].n_x)
					m_t_full_on_window.n_x = m_p_monitor[i].n_x;
				if(m_t_full_on_window.n_y > m_p_monitor[i].n_y)
					m_t_full_on_window.n_y = m_p_monitor[i].n_y;
				if(m_t_full_on_window.n_width < m_p_monitor[i].n_x + m_p_monitor[i].n_width)
					m_t_full_on_window.n_width = m_p_monitor[i].n_x + m_p_monitor[i].n_width;
				if(m_t_full_on_window.n_height < m_p_monitor[i].n_y + m_p_monitor[i].n_height)
					m_t_full_on_window.n_height = m_p_monitor[i].n_y + m_p_monitor[i].n_height;
			}
			// looking for min / max corner coords
		}
	}
	// find out what are the other monitors

	m_t_full_on_window.n_width -= m_t_full_on_window.n_x;
	m_t_full_on_window.n_height -= m_t_full_on_window.n_y;
	// we were looking for minimal / maximal corner coordinates, we want size

	strcpy(m_t_full_on_window.p_s_name, "virtual screen");
	// we rewrote this

	return true;
}

int TMultimonInfo::n_Monitor_Num() const
{
	return m_n_monitor_num;
}

const TMonitorInfo *TMultimonInfo::p_Monitor(int n_index) const
{
	return m_p_monitor + n_index;
}

const TMonitorInfo &TMultimonInfo::t_Full_Window(bool b_include_shutdown_monitors) const
{
	return (b_include_shutdown_monitors)? m_t_full_window : m_t_full_on_window;
}

BOOL CALLBACK TMultimonInfo::MonCallback(HMONITOR h_monitor, HDC h_dc, LPRECT p_rect, LPARAM n_l_param)
{
	TMultimonInfo *p_this = (TMultimonInfo*)n_l_param;

	for(int i = 0; i < p_this->m_n_monitor_num; i ++) {
		if(!p_this->m_p_monitor[i].b_valid) {
			MONITORINFOEX t_mon_info;
			memset(&t_mon_info, 0, sizeof(MONITORINFOEX));
			t_mon_info.cbSize = sizeof(MONITORINFOEX);
			if(!GetMonitorInfo(h_monitor, &t_mon_info)) {
				memset(&p_this->m_p_monitor[i], 0, sizeof(TMonitorInfo));
				p_this->m_p_monitor[i].b_on = false;
			} else {
				p_this->m_p_monitor[i].b_on = true;
				p_this->m_p_monitor[i].n_x = t_mon_info.rcMonitor.left;
				p_this->m_p_monitor[i].n_y = t_mon_info.rcMonitor.top;
				p_this->m_p_monitor[i].n_width = t_mon_info.rcMonitor.right -
					t_mon_info.rcMonitor.left;
				p_this->m_p_monitor[i].n_height = t_mon_info.rcMonitor.bottom -
					t_mon_info.rcMonitor.top;
				strncpy(p_this->m_p_monitor[i].p_s_name, (char*)t_mon_info.szDevice, 128);
			}
			p_this->m_p_monitor[i].b_valid = true;
			break;
		}
	}

	return true;
}
